(function ($, Drupal, drupalSettings, CKEDITOR){
  'use strict';

  CKEDITOR.plugins.add('countup',{
    icons: 'icon',
    hidpi: false,

    beforeInit: function (editor) {
      editor.widgets.add('countup', {
        allowedContent: 'span[class,data-start,data-end,data-duration,data-decimals,data-easing,data-grouping,data-separator,data-type-decimal,data-prefix,data-suffix]',
        requiredContent: 'span[class,data-start,data-end,data-duration,data-decimals,data-easing,data-grouping,data-separator,data-type-decimal,data-prefix,data-suffix]',

        upcast: function (element) {
          if (element.attributes['data-end'] === undefined) {
            return;
          }
          return element;
        },

        downcast: function (element) {
          return element;
        }
      });
    },

    init: function init(editor) {
      editor.addCommand('countup', {
        //allowedContent: countupAllowedContent,
        allowedContent: 'span[class,data-start,data-end,data-duration,data-decimals,data-easing,data-grouping,data-separator,data-type-decimal,data-prefix,data-suffix]',
        //requiredContent: 'span[class,data-start,data-end,data-duration,data-decimals,data-easing,data-grouping,data-separator,data-type-decimal,data-prefix,data-suffix]',
        modes: { wysiwyg: 1 },
        canUndo: true,
        exec: function exec(editor, data) {
          data = data || {};

          var existingValues = {};
          var selectedElement = editor.getSelection().getSelectedElement();

          if (selectedElement && selectedElement.$ && selectedElement.$.firstChild) {
            var widget = editor.widgets.getByElement(selectedElement, true);
            if (widget && widget.name === 'countup') {
              existingValues = selectedElement.$.firstChild.dataset;
            }
          }

          var dialogSettings = {
            title: editor.config.CountUp_dialogTitleAdd,
            dialogClass: 'countup-count-dialog'
          };

          if (existingValues) {
            dialogSettings.title = editor.config.CountUp_dialogTitleEdit;
          }

          var saveCallback = function saveCallback(returnValues) {
            editor.fire('saveSnapshot');
            var container = new CKEDITOR.dom.element('span', editor.document);

            container.addClass('countup');
            container.setAttributes(returnValues.attributes);

            if (returnValues.attributes['data-prefix'] !== undefined) {
              container.appendHtml(returnValues.attributes['data-prefix'])
            }

            container.appendHtml(returnValues.attributes['data-end']);

            if (returnValues.attributes['data-suffix'] !== undefined) {
              container.appendHtml(returnValues.attributes['data-suffix'])
            }

            editor.insertElement(container);
            editor.fire('saveSnapshot');
          };
          Drupal.ckeditor.openDialog(editor, Drupal.url('countup/dialog/count/' + editor.config.drupal.format), existingValues, saveCallback, dialogSettings);
        }
      });
      if (editor.ui.addButton){
        editor.ui.addButton('countup',{
          label: Drupal.t('Count'),
          command: 'countup',
          icon: this.path +'/icons/icon.png'
        });
      }
    }
  });

})(jQuery, Drupal, drupalSettings, CKEDITOR);
